#ifndef id753CAC41D3AE48A1A1D1EC399FD3DADF
#define id753CAC41D3AE48A1A1D1EC399FD3DADF

#include "doorkeeper/primitivetypes.hpp"
#include "doorkeeper/implem/vector.hpp"
#include <cstdint>
#include <cstddef>
#include <algorithm>
#include <limits>
#include <type_traits>

namespace dk {
	template <uint32_t D>
	class TileCoords {
		static_assert(D >= 1, "Invalid dimension");
		static_assert(D <= static_cast<std::size_t>(-1), "Dimension is too large");
	public:
		typedef Vector<D> coords;

		TileCoords ( void ) = default;
		TileCoords ( const TileCoords& ) = default;
		explicit TileCoords ( const coords& parSize );
		TileCoords ( const coords& parValue, const coords& parSize );

		TileCoords& operator++ ( void ); //pre
		TileCoords operator++ ( int ); //post
		TileCoords& operator-- ( void ); //pre
		TileCoords operator-- ( int ); //post
		const TileCoords& operator+= ( CoordinateScalarType parValue );
		const TileCoords& operator-= ( CoordinateScalarType parValue );
		bool operator== ( const TileCoords& parOther ) const;
		bool operator!= ( const TileCoords& parOther ) const;
		TileCoords& operator= ( const TileCoords& ) = default;
		const CoordinateScalarType& operator[] ( int parIndex ) const;
		CoordinateScalarType& operator[] ( int parIndex );

		const coords& position ( void ) const;
		const coords& upper ( void ) const;

	private:
		coords m_pos;
		coords m_size;
	};

	namespace implem {
		CoordinateScalarType sum_mod ( CoordinateScalarType parA, CoordinateScalarType parB, CoordinateScalarType parDiv ) a_pure;
		CoordinateScalarType sum_div ( CoordinateScalarType parA, CoordinateScalarType parB, CoordinateScalarType parDiv ) a_pure;
		CoordinateScalarType sub_mod ( CoordinateScalarType parA, CoordinateScalarType parB, CoordinateScalarType parDiv ) a_pure;
		CoordinateScalarType sub_div ( CoordinateScalarType parA, CoordinateScalarType parB, CoordinateScalarType parDiv ) a_pure;
	} //namespace implem

	template <uint32_t D>
	CoordinateDistType to_index ( const Vector<D>& parPos, const Vector<D>& parUpper ) a_pure;

	template <uint32_t D>
	CoordinateDistType to_index ( const TileCoords<D>& parTC ) a_pure;
} //namespace dk

#include "doorkeeper/implem/tilecoords.inl"

#endif
