#ifndef id873715F57B504CCF8227CE03EA28CAFA
#define id873715F57B504CCF8227CE03EA28CAFA

#include "doorkeeper/primitivetypes.hpp"
#include "doorkeeper/implem/helpers.hpp"
#include "doorkeeper/components/tilecoords.hpp"
#include <boost/iterator/iterator_facade.hpp>
#include <iterator>
#include <vector>
#include <type_traits>
#include <cstdint>
#include <utility>

namespace dk {
	template <typename T, uint32_t D, typename T1>
	class TileIterator;

	namespace implem {
		template <typename I, typename O>
		struct TypeWithQualifiers {
			typedef typename std::conditional<std::is_volatile<I>::value && std::is_const<I>::value,
						typename std::add_cv<O>::type,
						typename std::conditional<std::is_volatile<I>::value,
							typename std::add_volatile<typename std::remove_cv<O>::type>::type,
							typename std::conditional<std::is_const<I>::value,
								typename std::add_const<typename std::remove_cv<O>::type>::type,
								typename std::remove_cv<O>::type
							>::type
						>::type
					>::type result;
		};
	} //namespace implem

	template <typename T, uint32_t D, typename T1=typename std::remove_cv<T>::type>
	class TileIterator : public boost::iterator_facade<TileIterator<T, D>, T, boost::bidirectional_traversal_tag> {
		friend class boost::iterator_core_access;
		typedef std::vector<T1> vector_type;
		typedef typename implem::TypeWithQualifiers<T, vector_type>::result qualif_vector_type;
	public:
		typedef TileCoords<D> TileCoordsType;
		typedef typename TileCoordsType::coords coords;

		TileIterator ( void ) = default;
		TileIterator ( const TileIterator& parOther ) = default;
		TileIterator ( TileIterator&& parOther ) = default;
		TileIterator ( qualif_vector_type* parData, const coords& parArea );
		TileIterator ( qualif_vector_type* parData, const coords& parStart, const coords& parArea );
		~TileIterator ( void ) = default;

		const TileCoordsType& raw_coords ( void ) const;
		const coords& position ( void ) const;

	private:
		void increment ( void );
		void decrement ( void );
		void advance ( size_t parAdvance );
		ptrdiff_t distance_to ( const TileIterator& parOther );
		bool equal ( const TileIterator& parOther ) const;
		T& dereference ( void ) const;

		TileCoordsType m_tile_range;;
		qualif_vector_type* m_data;
	};

	template <uint32_t D>
	Vector<D> make_past_end_coords ( const Vector<D>& parTo ) a_pure;
} //namespace dk

#include "doorkeeper/implem/tileiterator.inl"

#endif
